(in-package :om)

;;; have make-a-measure return complete measures, according to measure-duration passed in call:

(defun make-a-measure (voice-ms tempo sign-denom measure-duration)
  (declare (ignore tempo))
  (list (read-from-string (format nil "~A//~A" measure-duration sign-denom))
	(let* ((beatsum (reduce '+ voice-ms
				:key #'(lambda (x) (if (numberp x) (round (abs x)) 1))))
	       (diff (- (- measure-duration beatsum))))
	  (if (< beatsum measure-duration)
	      (append voice-ms (list diff))
	      voice-ms))))

(defun quant-edit (durs tempi measures max/
                   &optional
                     (forbid nil)
                     (offset 0)
                     (autom 1))
  " Quantizes a list of <durs> (100 = 1 sec.) into the given measure(s),
with the given <tempi>.
<max/> is the maximum unit division that is taken to be a
significant duration.
A list of forbidden <forbid> unit divisions can optionally be
specified. The output is a list of
'measure-objects' that can be entered directly into an rtm-
box."
  (setf max/ (reduit-max max/))							  ;by AAA
  (let* ((tempos (if (= autom 2) (select-tempo durs) (expand-lst (list! tempi)))) ;(expand-lists (list! tempi))))
         (measures-x (expand-lst measures))
         (measures (if (consp (car measures-x)) measures-x (list measures-x)))
         (def-measure (cons (first (car (last measures))) (second (car (last measures)))))
         (durs (if (zerop offset)
		   durs
		   (cons (- (* offset (get-beat-duration (car tempos) (second (car measures)))))
                         durs)))
         (positive-durs (om-abs durs))
         (atimes (dx->x 0.0 positive-durs))
	 (c-time 0)
         (deftempo (car (last tempos)))
         result slur-fl current-tempo current-unit
	 current-measure-dur
         (max-list (and max/ (if (or (null (first max/)) (consp (first max/))) max/ (list max/))))
         (max-preci (and *distance-weight*
			 (if (or (null (first *distance-weight*)) (consp (first *distance-weight*)))
                             *distance-weight*
			     (list *distance-weight*))))
         (forbids (and forbid (if (or (null (first forbid)) (consp (first forbid)))
				  forbid
				  (list forbid))))
         (*unquantized-notes* 0)
	 (measure-number -1)
         (def-forbid (first (last forbids)))
         (def-max (first (last max/)))
         (def-preci (first (last *distance-weight*)))
         *global-grace-notes*
	 old-silence
	 (*max-division* max/))
    
    (reset-error)
    
    (do ()

	;;make new measures while:
	;;((or (null atimes) (>= c-time total-dur)))
	((or (null atimes) (>= c-time (last-elem atimes))))
      
	;;((null (cdr atimes)))				    ;dx->x above adds extra item

      (setf *max-division* (or (nth (+ 1 measure-number) max-list) def-max))
      (setf *distance-weight* (or (nth (+ 1 measure-number) max-preci) def-preci))
      (multiple-value-bind (beats times slur? current-time )
          (get-rhythms atimes
		       :tempo (setq current-tempo (or (pop tempos) deftempo))
                       :sign (if (car measures)
                                 (let ((this-measure (pop measures)))
				   (cons (setq current-measure-dur (first this-measure))
					 (setq current-unit (second this-measure))))
                                 (progn (setq current-unit (cdr def-measure))
					def-measure))
                       :start-time c-time
		       :forbid (or (nth (incf measure-number) forbids) def-forbid)
                       :old-slur? slur-fl)

	(setq atimes times)
	(setq c-time current-time)
        (setq slur-fl slur?)
        (if beats
            
	    (multiple-value-bind (beats-with-silences modifs new-silence)
                (put-in-silences beats durs old-silence)
              (setq old-silence new-silence)
              (when beats-with-silences
		(push (make-a-measure  beats-with-silences current-tempo current-unit current-measure-dur) result)       
		(setq durs (nthcdr modifs durs))))
	    
            (setq atimes nil))))

    (unless (zerop *unquantized-notes*)
      (if result
          (when *om-verbose* 
            (format *om-stream*
                    "Warning: with the given constraints, ~D notes are lost while quantizing ~%"
                    *unquantized-notes*))
          (om-beep-msg "cannot quantize with the given constraints")))
    (setq result (nreverse result))
    (list '?  result)))
